@@ -497,9 +497,9 @@ void (SET_HASHVALUE)(SEXP x, int v);
 #endif
 
 /* Pointer Protection and Unprotection */
-#define PROTECT(s)	protect(s)
-#define UNPROTECT(n)	unprotect(n)
-#define UNPROTECT_PTR(s)	unprotect_ptr(s)
+#define PROTECT(s)	Rf_protect(s)
+#define UNPROTECT(n)	Rf_unprotect(n)
+#define UNPROTECT_PTR(s)	Rf_unprotect_ptr(s)
 
 /* We sometimes need to coerce a protected value and place the new
    coerced value under protection.  For these cases PROTECT_WITH_INDEX
@@ -620,7 +620,7 @@ SEXP Rf_install(const char *);
 Rboolean Rf_isFree(SEXP);
 Rboolean Rf_isOrdered(SEXP);
 Rboolean Rf_isUnordered(SEXP);
-Rboolean Rf_isUnsorted(SEXP);
+Rboolean Rf_isUnsorted(SEXP, Rboolean);
 SEXP Rf_lengthgets(SEXP, R_len_t);
 SEXP R_lsInternal(SEXP, Rboolean);
 SEXP Rf_match(SEXP, SEXP, int);
@@ -665,6 +665,7 @@ typedef enum {
 
 cetype_t Rf_getCharCE(SEXP);
 SEXP Rf_mkCharCE(const char *, cetype_t);
+SEXP Rf_mkCharLenCE(const char *, int, cetype_t);
 const char *Rf_reEnc(const char *x, cetype_t ce_in, cetype_t ce_out, int subst);
 
 				/* return(.) NOT reached : for -Wall */
@@ -840,10 +841,6 @@ FILE *R_popen(const char *, const char *);
 #endif
 int R_system(const char *);
 
-/* now a macro */
-#define allocString(n)		Rf_allocVector(CHARSXP, n)
-#define Rf_allocString(n)      	Rf_allocVector(CHARSXP, n)
-
 #ifndef R_NO_REMAP
 #define acopy_string		Rf_acopy_string
 #define alloc3DArray            Rf_alloc3DArray
@@ -952,6 +949,7 @@ int R_system(const char *);
 #define mkChar			Rf_mkChar
 #define mkCharCE		Rf_mkCharCE
 #define mkCharLen		Rf_mkCharLen
+#define mkCharLenCE		Rf_mkCharLenCE
 #define mkString		Rf_mkString
 #define namesgets		Rf_namesgets
 #define ncols			Rf_ncols
