@@ -101,13 +101,21 @@ typedef enum JIT_OPCODE {
 
     JIT_for_i,          // "for" with INTEGER loop variable
 
+    // opcodes for for rhs expressions like x[i]
     // e.g. JIT_subset_r_i1 is x[i] where x is a real vec and i is int scalar.
     // Currently we only support indexes of length 1, so not JIT_subset_r_r.
 
-    JIT_subset_r_r, JIT_subset_r_r1,
-    JIT_subset_r_i, JIT_subset_r_i1,
-    JIT_subset_i_r, JIT_subset_i_r1,
-    JIT_subset_i_i, JIT_subset_i_i1,
+    JIT_subset_r_r1,
+    JIT_subset_r_i1,
+    JIT_subset_i_r1,
+    JIT_subset_i_i1,
+
+    // opcodes for for rhs expressions like x[i,j]
+
+    JIT_subset2_r_r1_r1, JIT_subset2_r_r1_i1,
+    JIT_subset2_r_i1_r1, JIT_subset2_r_i1_i1,
+    JIT_subset2_i_r1_r1, JIT_subset2_i_r1_i1,
+    JIT_subset2_i_i1_r1, JIT_subset2_i_i1_i1,
 
     // we generate assign ops only when both sides have the same length
 
@@ -269,10 +277,15 @@ static const char *JIT_OPCODE_NAMES[] = {
 
     "JIT_for_i",
 
-    "JIT_subset_r_r", "JIT_subset_r_r1",
-    "JIT_subset_r_i", "JIT_subset_r_i1",
-    "JIT_subset_i_r", "JIT_subset_i_r1",
-    "JIT_subset_i_i", "JIT_subset_i_i1",
+    "JIT_subset_r_r1",
+    "JIT_subset_r_i1",
+    "JIT_subset_i_r1",
+    "JIT_subset_i_i1",
+
+    "JIT_subset2_r_r1_r1", "JIT_subset2_r_r1_i1",
+    "JIT_subset2_r_i1_r1", "JIT_subset2_r_i1_i1",
+    "JIT_subset2_i_r1_r1", "JIT_subset2_i_r1_i1",
+    "JIT_subset2_i_i1_r1", "JIT_subset2_i_i1_i1",
 
     "JIT_as_r_r",  "JIT_as_r_r1",
     "JIT_as_r1_r", "JIT_as_r1_r1",
@@ -463,6 +476,7 @@ void genjitRealBin(ARITHOP_TYPE oper, CSEXP x, CSEXP y);
 void genjitRelop(RELOP_TYPE op, CSEXP x, CSEXP y);
 void genjitSubas(CSEXP x, CSEXP index, CSEXP y);
 void genjitSubset(CSEXP x, CSEXP subset);
+void genjitSubset2(CSEXP x, CSEXP subset);
 void genjitUnary(JIT_OPCODE opcode, CSEXP x);
 void initJit(void);
 void jitContextChangedAux(int nest);
