@@ -86,10 +86,6 @@
 * writing this note.  I guess it needs a bit more thought ...
 */
 
-/* <UTF8> char here is either ASCII or handled as a whole.
-   E.g. backquotify should work.
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -301,8 +297,14 @@ SEXP attribute_hidden do_dput(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (ifile != 1) {
 	con = getConnection(ifile);
 	wasopen = con->isopen;
-	if(!wasopen)
+	if(!wasopen) {
 	    if(!con->open(con)) error(_("cannot open the connection"));
+	    if(!con->canwrite) {
+		con->close(con);
+		error(_("cannot write to this connection"));
+	    }
+	} else if(!con->canwrite)
+	    error(_("cannot write to this connection"));
     }/* else: "Stdout" */
     for (i = 0; i < LENGTH(tval); i++)
 	if (ifile == 1)
@@ -375,8 +377,14 @@ SEXP attribute_hidden do_dump(SEXP call, SEXP op, SEXP args, SEXP rho)
 	else {
 	    con = getConnection(INTEGER(file)[0]);
 	    wasopen = con->isopen;
-	    if (!wasopen)
+	    if(!wasopen) {
 		if(!con->open(con)) error(_("cannot open the connection"));
+		if(!con->canwrite) {
+		    con->close(con);
+		    error(_("cannot write to this connection"));
+		}
+	    } else if(!con->canwrite)
+		error(_("cannot write to this connection"));
 	    for (i = 0, nout = 0; i < nobjs; i++) {
 		const char *s;
 		if (CAR(o) == R_UnboundValue) continue;
