@@ -19,8 +19,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 #undef HASHING
 
 #ifdef HAVE_CONFIG_H
@@ -79,7 +77,7 @@ SEXP applyClosure(SEXP call, SEXP op, SEXP arglist, SEXP rho, SEXP suppliedenv)
 
     f = formals;
     a = actuals;
-    while (f != RNIL) {
+    while (f != R_NilValue) {
         if (CAR(a) == R_MissingArg && CAR(f) != R_MissingArg) {
             SETCAR(a, mkPROMISE(CAR(f), newrho));
             SET_MISSING(a, 2);
@@ -90,12 +88,12 @@ SEXP applyClosure(SEXP call, SEXP op, SEXP arglist, SEXP rho, SEXP suppliedenv)
 
     /*  Fix up any extras that were supplied by usemethod. */
 
-    if (suppliedenv != RNIL) {
-        for (tmp = FRAME(suppliedenv); tmp != RNIL; tmp = CDR(tmp)) {
-            for (a = actuals; a != RNIL; a = CDR(a))
+    if (suppliedenv != R_NilValue) {
+        for (tmp = FRAME(suppliedenv); tmp != R_NilValue; tmp = CDR(tmp)) {
+            for (a = actuals; a != R_NilValue; a = CDR(a))
                 if (TAG(a) == TAG(tmp))
                     break;
-            if (a == RNIL)
+            if (a == R_NilValue)
                 /* Use defineVar instead of earlier version that added
                    bindings manually */
                 defineVar(TAG(tmp), CAR(tmp), newrho);
@@ -120,7 +118,7 @@ SEXP applyClosure(SEXP call, SEXP op, SEXP arglist, SEXP rho, SEXP suppliedenv)
     /* The default return value is NULL.  FIXME: Is this really needed
        or do we always get a sensible value returned?  */
 
-    tmp = RNIL;
+    tmp = R_NilValue;
 
     /* Debugging */
 
@@ -177,7 +175,7 @@ SEXP applyClosure(SEXP call, SEXP op, SEXP arglist, SEXP rho, SEXP suppliedenv)
     if ((SETJMP(cntxt.cjmpbuf))) {
         if (R_ReturnedValue == R_RestartToken) {
             cntxt.callflag = CTXT_RETURN;  /* turn restart off */
-            R_ReturnedValue = RNIL;  /* remove restart token */
+            R_ReturnedValue = R_NilValue;  /* remove restart token */
             PROTECT(tmp = eval(body, newrho));
         }
         else
@@ -213,7 +211,7 @@ static SEXP R_execClosure(SEXP call, SEXP op, SEXP arglist, SEXP rho,
     /* The default return value is NULL.  FIXME: Is this really needed
        or do we always get a sensible value returned?  */
 
-    tmp = RNIL;
+    tmp = R_NilValue;
 
     /* Debugging */
 
@@ -264,7 +262,7 @@ static SEXP R_execClosure(SEXP call, SEXP op, SEXP arglist, SEXP rho,
     if ((SETJMP(cntxt.cjmpbuf))) {
         if (R_ReturnedValue == R_RestartToken) {
             cntxt.callflag = CTXT_RETURN;  /* turn restart off */
-            R_ReturnedValue = RNIL;  /* remove restart token */
+            R_ReturnedValue = R_NilValue;  /* remove restart token */
             PROTECT(tmp = eval(body, newrho));
         }
         else
@@ -309,7 +307,7 @@ SEXP R_execMethod(SEXP op, SEXP rho)
 
     /* create a new environment frame enclosed by the lexical
        environment of the method */
-    PROTECT(newrho = Rf_NewEnvironment(RNIL, RNIL, CLOENV(op)));
+    PROTECT(newrho = Rf_NewEnvironment(R_NilValue, R_NilValue, CLOENV(op)));
 
     /* copy the bindings for the formal environment from the top frame
        of the internal environment of the generic call to the new
@@ -317,7 +315,7 @@ SEXP R_execMethod(SEXP op, SEXP rho)
        and the environments for any default expression promises are
        set to the new environment.  should move this to envir.c where
        it can be done more efficiently. */
-    for (next = FORMALS(op); next != RNIL; next = CDR(next)) {
+    for (next = FORMALS(op); next != R_NilValue; next = CDR(next)) {
         SEXP symbol =  TAG(next);
         R_varloc_t loc;
         int missing;
@@ -336,11 +334,11 @@ SEXP R_execMethod(SEXP op, SEXP rho)
                 SET_PRENV(val, newrho);
                 /* find the symbol in the method, copy its expression
                  * to the promise */
-                for(deflt = CAR(op); deflt != RNIL; deflt = CDR(deflt)) {
+                for(deflt = CAR(op); deflt != R_NilValue; deflt = CDR(deflt)) {
                     if(TAG(deflt) == symbol)
                         break;
                 }
-                if(deflt == RNIL)
+                if(deflt == R_NilValue)
                     error(_("symbol \"%s\" not in environment of method"),
                           CHAR(PRINTNAME(symbol)));
                 SET_PRCODE(val, CAR(deflt));
@@ -419,7 +417,7 @@ static SEXP replaceCall(SEXP fun, SEXP val, SEXP args, SEXP rhs)
     UNPROTECT(4);
     SETCAR(ptmp, fun); ptmp = CDR(ptmp);
     SETCAR(ptmp, val); ptmp = CDR(ptmp);
-    while(args != RNIL) {
+    while(args != R_NilValue) {
         SETCAR(ptmp, CAR(args));
         SET_TAG(ptmp, TAG(args));
         ptmp = CDR(ptmp);
@@ -454,11 +452,11 @@ SEXP attribute_hidden do_paren(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 SEXP attribute_hidden do_begin(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP s = RNIL;
-    if (args != RNIL) {
+    SEXP s = R_NilValue;
+    if (args != R_NilValue) {
         if (jitCompiling())
             decJitUnresolved(1);
-        while (args != RNIL) {
+        while (args != R_NilValue) {
             if (DEBUG(rho)) {
                 Rprintf("debug: ");
                 PrintValue(CAR(args));
@@ -495,14 +493,14 @@ SEXP attribute_hidden do_return(SEXP call, SEXP op, SEXP args, SEXP rho)
     }
     switch(nv) {
     case 0:
-        v = RNIL;
+        v = R_NilValue;
         break;
     case 1:
         v = CAR(vals);
         break;
     default:
         warningcall(call, _("multi-argument returns are deprecated"));
-        for (v = vals; v != RNIL; v = CDR(v)) {
+        for (v = vals; v != R_NilValue; v = CDR(v)) {
             if (CAR(v) == R_MissingArg)
                 errorcall(call, _("empty expression in return value"));
             if (NAMED(CAR(v)))
@@ -515,7 +513,7 @@ SEXP attribute_hidden do_return(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     findcontext(CTXT_BROWSER | CTXT_FUNCTION, rho, v);
 
-    return RNIL; /*NOTREACHED*/
+    return R_NilValue; /*NOTREACHED*/
 }
 
 
@@ -581,7 +579,7 @@ static SEXP evalseq(SEXP expr, SEXP rho, int forcelocal,  R_varloc_t tmploc)
         return CONS(nval, val);
     }
     else error(_("target of assignment expands to non-language object"));
-    return RNIL;        /*NOTREACHED*/
+    return R_NilValue;        /*NOTREACHED*/
 }
 
 /* Main entry point for complex assignments */
@@ -627,13 +625,13 @@ static SEXP applydefine(SEXP call, SEXP op, SEXP args, SEXP rho)
         errorcall(call, _("cannot do complex assignments in base namespace"));
     if (rho == R_BaseEnv)
         errorcall(call, _("cannot do complex assignments in base environment"));
-    defineVar(R_TmpvalSymbol, RNIL, rho);
+    defineVar(R_TmpvalSymbol, R_NilValue, rho);
     tmploc = R_findVarLocInFrame(rho, R_TmpvalSymbol);
     /* Now set up a context to remove it when we are done, even in the
      * case of an error.  This all helps error() provide a better call.
      */
     begincontext(&cntxt, CTXT_CCODE, call, R_BaseEnv, R_BaseEnv,
-                 RNIL, RNIL);
+                 R_NilValue, R_NilValue);
     cntxt.cend = &tmp_cleanup;
     cntxt.cenddata = rho;
 
@@ -768,7 +766,7 @@ SEXP attribute_hidden do_set(SEXP call, SEXP op, SEXP args, SEXP rho)
         UNIMPLEMENTED("do_set");
 
     }
-    return RNIL;/*NOTREACHED*/
+    return R_NilValue;/*NOTREACHED*/
 }
 
 
@@ -785,49 +783,56 @@ SEXP attribute_hidden evalList(SEXP el, SEXP rho, SEXP op)
     SEXP ans, h, tail, orig = el;
     int n = 1;
 
-    PROTECT(ans = tail = CONS(RNIL, RNIL));
+    PROTECT(ans = tail = CONS(R_NilValue, R_NilValue));
 
-    while (el != RNIL) {
+    while (el != R_NilValue) {
 
-        /* If we have a ... symbol, we look to see what it is bound to.
-         * If its binding is Null (i.e. zero length)
-         *      we just ignore it and return the cdr with all its expressions evaluated;
-         * if it is bound to a ... list of promises,
-         *      we force all the promises and then splice
-         *      the list of resulting values into the return value.
-         * Anything else bound to a ... symbol is an error
-        */
-        if (CAR(el) == R_DotsSymbol) {
-            h = findVar(CAR(el), rho);
-            if (TYPEOF(h) == DOTSXP || h == RNIL) {
-                while (h != RNIL) {
-                    SETCDR(tail, CONS(eval(CAR(h), rho), RNIL));
-                    SET_TAG(CDR(tail), CreateTag(TAG(h)));
-                    tail = CDR(tail);
-                    h = CDR(h);
-                }
-            }
-            else if (h != R_MissingArg)
-                error(_("'...' used in an incorrect context"));
-        } else if (CAR(el) != R_MissingArg) {
-            SETCDR(tail, CONS(eval(CAR(el), rho), RNIL));
-            tail = CDR(tail);
-            SET_TAG(tail, CreateTag(TAG(el)));
-        } else { /* It was a missing element */
-            SEXP line = STRING_ELT(deparse1line(orig, 0), 0);
-            PROTECT(line);
-            if(op == RNIL)
-                error(_("element %d is empty;\n   the part of the args "
-                        "list of a builtin being evaluated was:\n   %s"),
-                      n, CHAR(line)+4);
-            else
-                error(_("element %d is empty;\n   the part of the args "
-                        "list of '%s' being evaluated was:\n   %s"),
-                      n, PRIMNAME(op), CHAR(line)+4);
-            UNPROTECT(1);
-        }
-        el = CDR(el);
-        n++;
+	/* If we have a ... symbol, we look to see what it is bound to.
+	 * If its binding is Null (i.e. zero length)
+	 *	we just ignore it and return the cdr with all its expressions evaluated;
+	 * if it is bound to a ... list of promises,
+	 *	we force all the promises and then splice
+	 *	the list of resulting values into the return value.
+	 * Anything else bound to a ... symbol is an error
+	*/
+	if (CAR(el) == R_DotsSymbol) {
+	    h = findVar(CAR(el), rho);
+	    if (TYPEOF(h) == DOTSXP || h == R_NilValue) {
+		while (h != R_NilValue) {
+		    SETCDR(tail, CONS(eval(CAR(h), rho), R_NilValue));
+		    SET_TAG(CDR(tail), CreateTag(TAG(h)));
+		    tail = CDR(tail);
+		    h = CDR(h);
+		}
+	    }
+	    else if (h != R_MissingArg)
+		error(_("'...' used in an incorrect context"));
+/* Uncomment the following to restore old behavior */
+/* #define OLDMISSING */
+#ifdef OLDMISSING
+	} else if (CAR(el) != R_MissingArg) {
+#else
+	} else if (!(CAR(el) == R_MissingArg ||
+                 (isSymbol(CAR(el)) && R_isMissing(CAR(el), rho)))) {
+#endif
+	    SETCDR(tail, CONS(eval(CAR(el), rho), R_NilValue));
+	    tail = CDR(tail);
+	    SET_TAG(tail, CreateTag(TAG(el)));
+	} else { /* It was a missing element */
+	    SEXP line = STRING_ELT(deparse1line(orig, 0), 0);
+	    PROTECT(line);
+	    if(op == R_NilValue)
+		error(_("element %d is empty;\n   the part of the args "
+			"list of a builtin being evaluated was:\n   %s"),
+		      n, CHAR(line)+4);
+	    else
+		error(_("element %d is empty;\n   the part of the args "
+			"list of '%s' being evaluated was:\n   %s"),
+		      n, PRIMNAME(op), CHAR(line)+4);
+	    UNPROTECT(1);
+	}
+	el = CDR(el);
+	n++;
     }
     UNPROTECT(1);
     return CDR(ans);
@@ -843,45 +848,50 @@ SEXP attribute_hidden evalListKeepMissing(SEXP el, SEXP rho)
 {
     SEXP ans, h, tail;
 
-    PROTECT(ans = tail = CONS(RNIL, RNIL));
+    PROTECT(ans = tail = CONS(R_NilValue, R_NilValue));
 
-    while (el != RNIL) {
+    while (el != R_NilValue) {
 
-        /* If we have a ... symbol, we look to see what it is bound to.
-         * If its binding is Null (i.e. zero length)
-         *      we just ignore it and return the cdr with all its expressions evaluated;
-         * if it is bound to a ... list of promises,
-         *      we force all the promises and then splice
-         *      the list of resulting values into the return value.
-         * Anything else bound to a ... symbol is an error
-        */
-        if (CAR(el) == R_DotsSymbol) {
-            h = findVar(CAR(el), rho);
-            if (TYPEOF(h) == DOTSXP || h == RNIL) {
-                while (h != RNIL) {
-                    if (CAR(h) == R_MissingArg)
-                        SETCDR(tail, CONS(R_MissingArg, RNIL));
-                    else
-                        SETCDR(tail, CONS(eval(CAR(h), rho), RNIL));
-                    SET_TAG(CDR(tail), CreateTag(TAG(h)));
-                    tail = CDR(tail);
-                    h = CDR(h);
-                }
-            }
-            else if(h != R_MissingArg)
-                error(_("'...' used in an incorrect context"));
-        }
-        else if (CAR(el) == R_MissingArg) {
-            SETCDR(tail, CONS(R_MissingArg, RNIL));
-            tail = CDR(tail);
-            SET_TAG(tail, CreateTag(TAG(el)));
-        }
-        else {
-            SETCDR(tail, CONS(eval(CAR(el), rho), RNIL));
-            tail = CDR(tail);
-            SET_TAG(tail, CreateTag(TAG(el)));
-        }
-        el = CDR(el);
+	/* If we have a ... symbol, we look to see what it is bound to.
+	 * If its binding is Null (i.e. zero length)
+	 *	we just ignore it and return the cdr with all its expressions evaluated;
+	 * if it is bound to a ... list of promises,
+	 *	we force all the promises and then splice
+	 *	the list of resulting values into the return value.
+	 * Anything else bound to a ... symbol is an error
+	*/
+	if (CAR(el) == R_DotsSymbol) {
+	    h = findVar(CAR(el), rho);
+	    if (TYPEOF(h) == DOTSXP || h == R_NilValue) {
+		while (h != R_NilValue) {
+		    if (CAR(h) == R_MissingArg)
+			SETCDR(tail, CONS(R_MissingArg, R_NilValue));
+		    else
+			SETCDR(tail, CONS(eval(CAR(h), rho), R_NilValue));
+		    SET_TAG(CDR(tail), CreateTag(TAG(h)));
+		    tail = CDR(tail);
+		    h = CDR(h);
+		}
+	    }
+	    else if(h != R_MissingArg)
+		error(_("'...' used in an incorrect context"));
+	}
+#ifdef OLDMISSING
+	else if (CAR(el) == R_MissingArg) {
+#else
+	else if (CAR(el) == R_MissingArg ||
+                 (isSymbol(CAR(el)) && R_isMissing(CAR(el), rho))) {
+#endif
+	    SETCDR(tail, CONS(R_MissingArg, R_NilValue));
+	    tail = CDR(tail);
+	    SET_TAG(tail, CreateTag(TAG(el)));
+	}
+	else {
+	    SETCDR(tail, CONS(eval(CAR(el), rho), R_NilValue));
+	    tail = CDR(tail);
+	    SET_TAG(tail, CreateTag(TAG(el)));
+	}
+	el = CDR(el);
     }
     UNPROTECT(1);
     return CDR(ans);
@@ -897,9 +907,9 @@ SEXP attribute_hidden promiseArgs(SEXP el, SEXP rho)
 {
     SEXP ans, h, tail;
 
-    PROTECT(ans = tail = CONS(RNIL, RNIL));
+    PROTECT(ans = tail = CONS(R_NilValue, R_NilValue));
 
-    while(el != RNIL) {
+    while(el != R_NilValue) {
 
         /* If we have a ... symbol, we look to see what it is bound to.
          * If its binding is Null (i.e. zero length)
@@ -914,9 +924,9 @@ SEXP attribute_hidden promiseArgs(SEXP el, SEXP rho)
 
         if (CAR(el) == R_DotsSymbol) {
             h = findVar(CAR(el), rho);
-            if (TYPEOF(h) == DOTSXP || h == RNIL) {
-                while (h != RNIL) {
-                    SETCDR(tail, CONS(mkPROMISE(CAR(h), rho), RNIL));
+            if (TYPEOF(h) == DOTSXP || h == R_NilValue) {
+                while (h != R_NilValue) {
+                    SETCDR(tail, CONS(mkPROMISE(CAR(h), rho), R_NilValue));
                     SET_TAG(CDR(tail), CreateTag(TAG(h)));
                     tail = CDR(tail);
                     h = CDR(h);
@@ -926,12 +936,12 @@ SEXP attribute_hidden promiseArgs(SEXP el, SEXP rho)
                 error(_("'...' used in an incorrect context"));
         }
         else if (CAR(el) == R_MissingArg) {
-            SETCDR(tail, CONS(R_MissingArg, RNIL));
+            SETCDR(tail, CONS(R_MissingArg, R_NilValue));
             tail = CDR(tail);
             SET_TAG(tail, CreateTag(TAG(el)));
         }
         else {
-            SETCDR(tail, CONS(mkPROMISE(CAR(el), rho), RNIL));
+            SETCDR(tail, CONS(mkPROMISE(CAR(el), rho), R_NilValue));
             tail = CDR(tail);
             SET_TAG(tail, CreateTag(TAG(el)));
         }
@@ -948,7 +958,7 @@ SEXP attribute_hidden promiseArgs(SEXP el, SEXP rho)
 void attribute_hidden CheckFormals(SEXP ls)
 {
     if (isList(ls)) {
-        for (; ls != RNIL; ls = CDR(ls))
+        for (; ls != R_NilValue; ls = CDR(ls))
             if (TYPEOF(TAG(ls)) != SYMSXP)
                 goto err;
         return;
@@ -987,14 +997,14 @@ SEXP attribute_hidden do_eval(SEXP call, SEXP op, SEXP args, SEXP rho)
         PROTECT(env);   /* so we can unprotect 2 at the end */
         break;
     case LISTSXP:
-        env = NewEnvironment(RNIL, duplicate(CADR(args)), encl);
+        env = NewEnvironment(R_NilValue, duplicate(CADR(args)), encl);
         PROTECT(env);
         break;
     case VECSXP:
         x = VectorToPairList(CADR(args));
-        for (xptr = x ; xptr != RNIL ; xptr = CDR(xptr))
+        for (xptr = x ; xptr != R_NilValue ; xptr = CDR(xptr))
             SET_NAMED(CAR(xptr) , 2);
-        env = NewEnvironment(RNIL, x, encl);
+        env = NewEnvironment(R_NilValue, x, encl);
         PROTECT(env);
         break;
     case INTSXP:
@@ -1032,7 +1042,7 @@ SEXP attribute_hidden do_eval(SEXP call, SEXP op, SEXP args, SEXP rho)
         int i, n;
         PROTECT(expr);
         n = LENGTH(expr);
-        tmp = RNIL;
+        tmp = R_NilValue;
         begincontext(&cntxt, CTXT_RETURN, call, env, rho, args, op);
         if (!SETJMP(cntxt.cjmpbuf))
             for(i = 0 ; i < n ; i++)
@@ -1094,7 +1104,7 @@ SEXP attribute_hidden do_recall(SEXP call, SEXP op, SEXP args, SEXP rho)
        otherwise search for it by name or evaluate the expression
        originally used to get it.
     */
-    if (cptr->callfun != RNIL)
+    if (cptr->callfun != R_NilValue)
         PROTECT(s = cptr->callfun);
     else if( TYPEOF(CAR(cptr->call)) == SYMSXP)
         PROTECT(s = findFun(CAR(cptr->call), cptr->sysparent));
@@ -1135,105 +1145,121 @@ int DispatchOrEval(SEXP call, SEXP op, const char *generic, SEXP args,
    might come in with a "..." and that there might be other arguments
    in the "..." as well.  LT */
 
-    SEXP x = RNIL;
+    SEXP x = R_NilValue;
     int dots = FALSE, nprotect = 0;;
 
     if( argsevald )
-        {PROTECT(x = CAR(args)); nprotect++;}
+	{PROTECT(x = CAR(args)); nprotect++;}
     else {
-        /* Find the object to dispatch on, dropping any leading
-           ... arguments with missing or empty values.  If there are no
-           arguments, RNIL is used. */
-        for (; args != RNIL; args = CDR(args)) {
-            if (CAR(args) == R_DotsSymbol) {
-                SEXP h = findVar(R_DotsSymbol, rho);
-                if (TYPEOF(h) == DOTSXP) {
-                    /* just a consistency check */
-                    if (TYPEOF(CAR(h)) != PROMSXP)
-                        error(_("value in '...' is not a promise"));
-                    dots = TRUE;
-                    x = eval(CAR(h), rho);
-                break;
-                }
-                else if (h != RNIL && h != R_MissingArg)
-                    error(_("'...' used in an incorrect context"));
-            }
-            else {
-                dots = FALSE;
-            x = eval(CAR(args), rho);
-            break;
-            }
-        }
-        PROTECT(x); nprotect++;
+	/* Find the object to dispatch on, dropping any leading
+	   ... arguments with missing or empty values.  If there are no
+	   arguments, R_NilValue is used. */
+	for (; args != R_NilValue; args = CDR(args)) {
+	    if (CAR(args) == R_DotsSymbol) {
+		SEXP h = findVar(R_DotsSymbol, rho);
+		if (TYPEOF(h) == DOTSXP) {
+		    /* just a consistency check */
+		    if (TYPEOF(CAR(h)) != PROMSXP)
+			error(_("value in '...' is not a promise"));
+		    dots = TRUE;
+		    x = eval(CAR(h), rho);
+		break;
+		}
+		else if (h != R_NilValue && h != R_MissingArg)
+		    error(_("'...' used in an incorrect context"));
+	    }
+	    else {
+		dots = FALSE;
+	    x = eval(CAR(args), rho);
+	    break;
+	    }
+	}
+	PROTECT(x); nprotect++;
     }
-        /* try to dispatch on the object */
+	/* try to dispatch on the object */
     if( isObject(x) ) {
-        char *pt;
-        /* Try for formal method. */
-        if(IS_S4_OBJECT(x) && R_has_methods(op)) {
-            SEXP value, argValue;
-            /* create a promise to pass down to applyClosure  */
-            if(!argsevald) {
-                argValue = promiseArgs(args, rho);
-                SET_PRVALUE(CAR(argValue), x);
-            } else argValue = args;
-            PROTECT(argValue); nprotect++;
-            /* This means S4 dispatch */
-            value = R_possible_dispatch(call, op, argValue, rho, TRUE);
-            if(value) {
-                *ans = value;
-                UNPROTECT(nprotect);
-                return 1;
-            }
-            else {
-                /* go on, with the evaluated args.  Not guaranteed to have
-                   the same semantics as if the arguments were not
-                   evaluated, in special cases (e.g., arg values that are
-                   LANGSXP).
-                   The use of the promiseArgs is supposed to prevent
-                   multiple evaluation after the call to possible_dispatch.
-                */
-                if (dots)
-                    argValue = evalArgs(argValue, rho, op, dropmissing);
-                else {
-                    argValue = CONS(x, evalArgs(CDR(argValue), rho, op, dropmissing));
-                    SET_TAG(argValue, CreateTag(TAG(args)));
-                }
-                PROTECT(args = argValue); nprotect++;
-                argsevald = 1;
-            }
-        }
-        if (TYPEOF(CAR(call)) == SYMSXP)
-            pt = Rf_strrchr(CHAR(PRINTNAME(CAR(call))), '.');
-        else
-            pt = NULL;
+	char *pt;
+	/* Try for formal method. */
+	if(IS_S4_OBJECT(x) && R_has_methods(op)) {
+	    SEXP value, argValue;
+	    /* create a promise to pass down to applyClosure  */
+	    if(!argsevald) {
+		argValue = promiseArgs(args, rho);
+		SET_PRVALUE(CAR(argValue), x);
+	    } else argValue = args;
+	    PROTECT(argValue); nprotect++;
+	    /* This means S4 dispatch */
+	    value = R_possible_dispatch(call, op, argValue, rho, TRUE);
+	    if(value) {
+		*ans = value;
+		UNPROTECT(nprotect);
+		return 1;
+	    }
+	    else {
+		/* go on, with the evaluated args.  Not guaranteed to have
+		   the same semantics as if the arguments were not
+		   evaluated, in special cases (e.g., arg values that are
+		   LANGSXP).
+		   The use of the promiseArgs is supposed to prevent
+		   multiple evaluation after the call to possible_dispatch.
+		*/
+		if (dots)
+		    argValue = evalArgs(argValue, rho, op, dropmissing);
+		else {
+		    argValue = CONS(x, evalArgs(CDR(argValue), rho, op, dropmissing));
+		    SET_TAG(argValue, CreateTag(TAG(args)));
+		}
+		PROTECT(args = argValue); nprotect++;
+		argsevald = 1;
+	    }
+	}
+	if (TYPEOF(CAR(call)) == SYMSXP)
+	    pt = Rf_strrchr(CHAR(PRINTNAME(CAR(call))), '.');
+	else
+	    pt = NULL;
 
-        if (pt == NULL || strcmp(pt,".default")) {
-            RCNTXT cntxt;
-            SEXP pargs;
-            PROTECT(pargs = promiseArgs(args, rho)); nprotect++;
-            SET_PRVALUE(CAR(pargs), x);
-            begincontext(&cntxt, CTXT_RETURN, call, rho, rho, pargs, op);
-            if(usemethod(generic, x, call, pargs, rho, rho, R_BaseEnv, ans))
-            {
-                endcontext(&cntxt);
-                UNPROTECT(nprotect);
-                return 1;
-            }
-            endcontext(&cntxt);
-        }
+	if (pt == NULL || strcmp(pt,".default")) {
+	    RCNTXT cntxt;
+	    SEXP pargs, rho1;
+	    PROTECT(pargs = promiseArgs(args, rho)); nprotect++;
+	    /* The context set up here is needed because of the way
+	       usemethod() is written.  DispatchGroup() repeats some
+	       internal usemethod() code and avoids the need for a
+	       context; perhaps the usemethod() code should be
+	       refactored so the contexts around the usemethod() calls
+	       in this file can be removed.
+
+	       Using rho for current and calling environment can be
+	       confusing for things like sys.parent() calls captured
+	       in promises (Gabor G had an example of this).  Also,
+	       since the context is established without a SETJMP using
+	       an R-accessible environment allows a segfault to be
+	       triggered (by something very obscure, but still).
+	       Hence here and in the other usemethod() uses below a
+	       new environment rho1 is created and used.  LT */
+	    PROTECT(rho1 = NewEnvironment(R_NilValue, R_NilValue, rho)); nprotect++;
+	    SET_PRVALUE(CAR(pargs), x);
+	    begincontext(&cntxt, CTXT_RETURN, call, rho1, rho, pargs, op);
+	    if(usemethod(generic, x, call, pargs, rho1, rho, R_BaseEnv, ans))
+	    {
+		endcontext(&cntxt);
+		UNPROTECT(nprotect);
+		return 1;
+	    }
+	    endcontext(&cntxt);
+	}
     }
     if(!argsevald) {
-        if (dots)
-            /* The first call argument was ... and may contain more than the
-               object, so it needs to be evaluated here.  The object should be
-               in a promise, so evaluating it again should be no problem. */
-            *ans = evalArgs(args, rho, op, dropmissing);
-        else {
-            PROTECT(*ans = CONS(x, evalArgs(CDR(args), rho, op, dropmissing)));
-            SET_TAG(*ans, CreateTag(TAG(args)));
-            UNPROTECT(1);
-        }
+	if (dots)
+	    /* The first call argument was ... and may contain more than the
+	       object, so it needs to be evaluated here.  The object should be
+	       in a promise, so evaluating it again should be no problem. */
+	    *ans = evalArgs(args, rho, op, dropmissing);
+	else {
+	    PROTECT(*ans = CONS(x, evalArgs(CDR(args), rho, op, dropmissing)));
+	    SET_TAG(*ans, CreateTag(TAG(args)));
+	    UNPROTECT(1);
+	}
     }
     else *ans = args;
     UNPROTECT(nprotect);
@@ -1293,8 +1319,8 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
        of the first two is an object -- both of these cases would be
        rejected by the code following the string examination code
        below */
-    if (args != RNIL && ! isObject(CAR(args)) &&
-        (CDR(args) == RNIL || ! isObject(CADR(args))))
+    if (args != R_NilValue && ! isObject(CAR(args)) &&
+        (CDR(args) == R_NilValue || ! isObject(CADR(args))))
         return 0;
 
     isOps = strcmp(group, "Ops") == 0;
@@ -1307,7 +1333,7 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
         SEXP value;
         /* Remove argument names to ensure positional matching */
         if(isOps)
-            for(s = args; s != RNIL; s = CDR(s)) SET_TAG(s, RNIL);
+            for(s = args; s != R_NilValue; s = CDR(s)) SET_TAG(s, R_NilValue);
 
         value = R_possible_dispatch(call, op, args, rho, FALSE);
         if(value) {
@@ -1349,10 +1375,10 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
     if( nargs == 2 )
         rclass = getAttrib(CADR(args), R_ClassSymbol);
     else
-        rclass = RNIL;
+        rclass = R_NilValue;
 
-    lsxp = RNIL; lgr = RNIL; lmeth = RNIL;
-    rsxp = RNIL; rgr = RNIL; rmeth = RNIL;
+    lsxp = R_NilValue; lgr = R_NilValue; lmeth = R_NilValue;
+    rsxp = R_NilValue; rgr = R_NilValue; rmeth = R_NilValue;
 
     findmethod(lclass, group, generic, &lsxp, &lgr, &lmeth, &lwhich,
                lbuf, rho);
@@ -1435,11 +1461,11 @@ int DispatchGroup(const char* group, SEXP call, SEXP op, SEXP args, SEXP rho,
 
     PROTECT(s = promiseArgs(CDR(call), rho));
     if (length(s) != length(args))
-        error(_("dispatch error"));
-    for (m = s ; m != RNIL ; m = CDR(m), args = CDR(args) ) {
+	error(_("dispatch error in group dispatch"));
+    for (m = s ; m != R_NilValue ; m = CDR(m), args = CDR(args) ) {
         SET_PRVALUE(CAR(m), CAR(args));
         /* ensure positional matching for operators */
-        if(isOps) SET_TAG(m, RNIL);
+        if(isOps) SET_TAG(m, R_NilValue);
     }
 
     *ans = applyClosure(t, lsxp, s, rho, newrho);
