@@ -509,16 +509,18 @@ typedef int BCODE;
 static int tryDispatch(char *generic, SEXP call, SEXP x, SEXP rho, SEXP *pv)
 {
   RCNTXT cntxt;
-  SEXP pargs;
+  SEXP pargs, rho1;
   int dispatched = FALSE;
 
   PROTECT(pargs = promiseArgs(CDR(call), rho));
+  /* See comment at first usemethod() call in this file. LT */
+  PROTECT(rho1 = NewEnvironment(R_NilValue, R_NilValue, rho));
   SET_PRVALUE(CAR(pargs), x);
-  begincontext(&cntxt, CTXT_RETURN, call, rho, rho, pargs, R_NilValue);/**** FIXME: put in op */
-  if (usemethod(generic, x, call, pargs, rho, rho, R_BaseEnv, pv))
+  begincontext(&cntxt, CTXT_RETURN, call, rho1, rho, pargs, R_NilValue);/**** FIXME: put in op */
+  if (usemethod(generic, x, call, pargs, rho1, rho, R_BaseEnv, pv))
     dispatched = TRUE;
   endcontext(&cntxt);
-  UNPROTECT(1);
+  UNPROTECT(2);
   return dispatched;
 }
 
@@ -1191,16 +1193,18 @@ SEXP bcEval(SEXP body, SEXP rho)
 	SEXP x = R_BCNodeStackTop[-1];
 	if (isObject(x)) {
 	  RCNTXT cntxt;
-	  SEXP pargs, str;
+	  SEXP pargs, str, rho1;
 	  PROTECT(pargs = promiseArgs(CDR(call), rho));
+	  /* See comment at first usemethod() call in this file. LT */
+	  PROTECT(rho1 = NewEnvironment(R_NilValue, R_NilValue, rho));
 	  SET_PRVALUE(CAR(pargs), x);
 	  str = ScalarString(PRINTNAME(symbol));
 	  SET_PRVALUE(CADR(pargs), str);
-	  begincontext(&cntxt, CTXT_RETURN, call, rho, rho, pargs, R_NilValue);/**** FIXME: put in op */
-	  if (usemethod("$", x, call, pargs, rho, rho, R_BaseEnv, &value))
+	  begincontext(&cntxt, CTXT_RETURN, call, rho1, rho, pargs, R_NilValue);/**** FIXME: put in op */
+	  if (usemethod("$", x, call, pargs, rho1, rho, R_BaseEnv, &value))
 	    dispatched = TRUE;
 	  endcontext(&cntxt);
-	  UNPROTECT(1);
+	  UNPROTECT(2);
 	}
 	if (dispatched)
 	  R_BCNodeStackTop[-1] = value;
@@ -1217,17 +1221,19 @@ SEXP bcEval(SEXP body, SEXP rho)
 	value = R_BCNodeStackTop[-2];
 	if (isObject(x)) {
 	  RCNTXT cntxt;
-	  SEXP pargs, str;
+	  SEXP pargs, str, rho1;
 	  PROTECT(pargs = promiseArgs(CDR(call), rho));
+	  /* See comment at first usemethod() call in this file. LT */
+	  PROTECT(rho1 = NewEnvironment(R_NilValue, R_NilValue, rho));
 	  SET_PRVALUE(CAR(pargs), x);
 	  str = ScalarString(PRINTNAME(symbol));
 	  SET_PRVALUE(CADR(pargs), str);
 	  SET_PRVALUE(CADDR(pargs), value);
-	  begincontext(&cntxt, CTXT_RETURN, call, rho, rho, pargs, R_NilValue);/**** FIXME: put in op */
-	  if (usemethod("$<-", x, call, pargs, rho, rho, R_BaseEnv, &value))
+	  begincontext(&cntxt, CTXT_RETURN, call, rho1, rho, pargs, R_NilValue);/**** FIXME: put in op */
+	  if (usemethod("$<-", x, call, pargs, rho1, rho, R_BaseEnv, &value))
 	    dispatched = TRUE;
 	  endcontext(&cntxt);
-	  UNPROTECT(1);
+	  UNPROTECT(2);
 	}
 	R_BCNodeStackTop--;
 	if (dispatched)
