@@ -40,13 +40,18 @@ int  istack;                        // index into exec stack
 #define TOSR (REAL(stack[istack]))
 #define TOSI (INTEGER(stack[istack]))
 
-// Defs for binops e.g. "x + y" puts x at [istack-1] and y at [istack].
+// Defs for binops and triops e.g. "x + y" puts x at [istack-1] and y at [istack].
 // Remember: "y is top of stack; x is top of stack minus 1".
 
+#define W  (stack[istack-2])
 #define X  (stack[istack-1])
 #define Y  (stack[istack])
+
+#define WR (REAL(stack[istack-2]))
 #define XR (REAL(stack[istack-1]))
 #define YR (REAL(stack[istack]))
+
+#define WI (INTEGER(stack[istack-2]))
 #define XI (INTEGER(stack[istack-1]))
 #define YI (INTEGER(stack[istack]))
 
@@ -299,6 +304,19 @@ static R_INLINE void DCheckSubscript_x_y1(SEXPTYPE xtype, SEXPTYPE ytype,
     DEvalAssert(LENGTH(result) == 1, NULL);
 }
 
+static R_INLINE void DCheckSubscript2_w_x1_y1(SEXPTYPE wtype, SEXPTYPE xtype, 
+                                             SEXPTYPE ytype, CSEXP result)
+{
+    DEvalAssert(istack >= 4, NULL);
+    DAssertCompatibleType(W, wtype);
+    DAssertCompatibleType(X, xtype);
+    DAssertCompatibleType(Y, ytype);
+    DAssertCompatibleType(result, wtype);
+    DEvalAssert(LENGTH(X) == 1, NULL);
+    DEvalAssert(LENGTH(Y) == 1, NULL);
+    DEvalAssert(LENGTH(result) == 1, NULL);
+}
+
 static R_INLINE void DCheckUminus(CSEXP result, SEXPTYPE xtype)
 {                                       // unary minus, vec arg
     DEvalAssert(istack >= 2, NULL);
@@ -378,25 +396,39 @@ static R_INLINE void drop2(const JIT_RECORD * const prec)
         stackUnderflowError(prec);
     istack -= 2;
 }
-static R_INLINE void CheckSubscriptRange_i(CSEXP x, int i,
+static R_INLINE int nrowsv(SEXP s)	// fast version of nrows(), vectors only
+{
+    Dassert(isVector(s));
+    CSEXP t = getAttrib(s, R_DimSymbol);
+    Dassert(t != R_NilValue);
+    return INTEGER(t)[0];
+}
+static R_INLINE int ncols_vec(SEXP s)	// fast version of ncols(), vectors only
+{
+    Dassert(isVector(s));
+    CSEXP t = getAttrib(s, R_DimSymbol);
+    Dassert(t != R_NilValue);
+    return INTEGER(t)[1];
+}
+static R_INLINE void CheckSubscriptRange_i(int nx, int i,
                                            const JIT_RECORD * const prec)
 {
-    if (i < 0 || i >= LENGTH(x)) {
+    if (i < 0 || i >= nx) {
         if (i == R_NaInt-1)     // subtract 1 to match 1 index offset
             error(_("integer NA index in jitted expression %s"),
                   deparseAsShortString(prec->original));
         else
             error(_("out of range index %d (allowed range is 1 to %d)\n"
                   "       in jitted expression %s"),
-                  i+1, LENGTH(x), deparseAsShortString(prec->original));
+                  i+1, nx, deparseAsShortString(prec->original));
     }
 }
-static R_INLINE void CheckSubscriptRange_r(CSEXP x, double i,
+static R_INLINE void CheckSubscriptRange_r(int nx, double i,
                                            const JIT_RECORD * const prec)
 {
     // test is "backwards" so NaNs are detected too (NaNs always compare false)
 
-    if (i >= 0 && i < LENGTH(x))
+    if (i >= 0 && i < nx)
         ;
     else if (isnan(i))
         error(_("NaN index in jitted expression %s"),
@@ -404,7 +436,7 @@ static R_INLINE void CheckSubscriptRange_r(CSEXP x, double i,
     else
         error(_("out of range index %g (allowed range is 1 to %d)\n"
               "       in jitted expression %s"),
-              i+1, LENGTH(x), deparseAsShortString(prec->original));
+              i+1, nx, deparseAsShortString(prec->original));
 }
 static R_INLINE double FMOD(double x, double y) // for R %% operator
 {
@@ -606,8 +638,8 @@ SEXP evalJit(CSEXP e)
     JIT_OP *op = p->ops;                // pointer to current jit op
     int istackStart = istack;
 
-    double r, *xr, *yr, *ar, xr0, yr0;  // temps for faster code
-    int i, n, *xi, *yi, *ai, xi0, yi0;
+    double r, *xr, *yr, *ar, wr0, xr0, yr0;  // temps for faster code
+    int i, n, *xi, *yi, *ai, wi0, xi0, yi0;
     FUNC_TYPE   func;
     IFUNC_TYPE  ifunc;
     SEXP result;
