@@ -206,28 +206,94 @@ case JIT_for_i:
 
 case JIT_subset_r_i1:
     result = op->result; DCheckSubscript_x_y1(REALSXP, INTSXP, result);
-    yi0 = YI[0] - 1; CheckSubscriptRange_i(X, yi0, p);
+    yi0 = YI[0] - 1; CheckSubscriptRange_i(LENGTH(X), yi0, p);
     xr0 = XR[yi0];  X = result; *REAL(stack[--istack]) = xr0;
     break;
 
 case JIT_subset_r_r1:
     result = op->result; DCheckSubscript_x_y1(REALSXP, REALSXP, result);
-    yr0 = YR[0] - 1; CheckSubscriptRange_r(X, yr0, p);
+    yr0 = YR[0] - 1; CheckSubscriptRange_r(LENGTH(X), yr0, p);
     xr0 = XR[(int)yr0]; X = result; *REAL(stack[--istack]) = xr0;
     break;
 
 case JIT_subset_i_i1:
     result = op->result; DCheckSubscript_x_y1(INTSXP, INTSXP, result);
-    yi0 = YI[0] - 1; CheckSubscriptRange_i(X, yi0, p);
+    yi0 = YI[0] - 1; CheckSubscriptRange_i(LENGTH(X), yi0, p);
     xi0 = XI[yi0];  X = result; *INTEGER(stack[--istack]) = xi0;
     break;
 
 case JIT_subset_i_r1:
     result = op->result; DCheckSubscript_x_y1(INTSXP, REALSXP, result);
-    yr0 = YR[0] - 1; CheckSubscriptRange_r(X, yr0, p);
+    yr0 = YR[0] - 1; CheckSubscriptRange_r(LENGTH(X), yr0, p);
     xi0 = XI[(int)yr0]; X = result; *INTEGER(stack[--istack]) = xi0;
     break;
 
+// x[index1,index2]: TOS2 W is x, TOS1 X is index, TOS Y is index2
+
+case JIT_subset2_r_r1_r1:
+    result = op->result; DCheckSubscript2_w_x1_y1(REALSXP, REALSXP, REALSXP, result);
+    n = nrowsv(W);
+    xr0 = XR[0] - 1; CheckSubscriptRange_r(n, xr0, p);
+    yr0 = YR[0] - 1; CheckSubscriptRange_r(ncols_vec(W), yr0, p);
+    wr0 = WR[(int)(yr0 * n + xr0)];  W = result; *REAL(stack[istack-=2]) = wr0;
+    break;
+
+case JIT_subset2_r_r1_i1:
+    result = op->result; DCheckSubscript2_w_x1_y1(REALSXP, REALSXP, INTSXP, result);
+    n = nrowsv(W);
+    xr0 = XR[0] - 1; CheckSubscriptRange_r(n, xr0, p);
+    yi0 = YI[0] - 1; CheckSubscriptRange_i(ncols_vec(W), yi0, p);
+    wr0 = WR[(int)(yi0 * n + xr0)];  W = result; *REAL(stack[istack-=2]) = wr0;
+    break;
+
+case JIT_subset2_r_i1_r1:
+    result = op->result; DCheckSubscript2_w_x1_y1(REALSXP, INTSXP, REALSXP, result);
+    n = nrowsv(W);
+    xi0 = XI[0] - 1; CheckSubscriptRange_i(n, xi0, p);
+    yr0 = YR[0] - 1; CheckSubscriptRange_r(ncols_vec(W), yr0, p);
+    wr0 = WR[(int)(yr0 * n + xi0)];  W = result; *REAL(stack[istack-=2]) = wr0;
+    break;
+
+case JIT_subset2_r_i1_i1:
+    result = op->result; DCheckSubscript2_w_x1_y1(REALSXP, INTSXP, INTSXP, result);
+    n = nrowsv(W);
+    xi0 = XI[0] - 1; CheckSubscriptRange_i(n, xi0, p);
+    yi0 = YI[0] - 1; CheckSubscriptRange_i(ncols_vec(W), yi0, p);
+    wr0 = WR[(int)(yi0 * n + xi0)];  W = result; *REAL(stack[istack-=2]) = wr0;
+    break;
+
+case JIT_subset2_i_r1_r1:
+    result = op->result; DCheckSubscript2_w_x1_y1(INTSXP, REALSXP, REALSXP, result);
+    n = nrowsv(W);
+    xr0 = XR[0] - 1; CheckSubscriptRange_r(n, xr0, p);
+    yr0 = YR[0] - 1; CheckSubscriptRange_r(ncols_vec(W), yr0, p);
+    wi0 = WI[(int)(yr0 * n + xr0)];  W = result; *INTEGER(stack[istack-=2]) = wi0;
+    break;
+
+case JIT_subset2_i_r1_i1:
+    result = op->result; DCheckSubscript2_w_x1_y1(INTSXP, REALSXP, INTSXP, result);
+    n = nrowsv(W);
+    xr0 = XR[0] - 1; CheckSubscriptRange_r(n, xr0, p);
+    yi0 = YI[0] - 1; CheckSubscriptRange_i(ncols_vec(W), yi0, p);
+    wi0 = WI[(int)(yi0 * n + xr0)];  W = result; *INTEGER(stack[istack-=2]) = wi0;
+    break;
+
+case JIT_subset2_i_i1_r1:
+    result = op->result; DCheckSubscript2_w_x1_y1(INTSXP, INTSXP, REALSXP, result);
+    n = nrowsv(W);
+    xi0 = XI[0] - 1; CheckSubscriptRange_i(n, xi0, p);
+    yr0 = YR[0] - 1; CheckSubscriptRange_r(ncols_vec(W), yr0, p);
+    wi0 = WI[(int)(yr0 * n + xi0)];  W = result; *INTEGER(stack[istack-=2]) = wi0;
+    break;
+
+case JIT_subset2_i_i1_i1:
+    result = op->result; DCheckSubscript2_w_x1_y1(INTSXP, INTSXP, INTSXP, result);
+    n = nrowsv(W);
+    xi0 = XI[0] - 1; CheckSubscriptRange_i(n, xi0, p);
+    yi0 = YI[0] - 1; CheckSubscriptRange_i(ncols_vec(W), yi0, p);
+    wi0 = WI[(int)(yi0 * n + xi0)];  W = result; *INTEGER(stack[istack-=2]) = wi0;
+    break;
+
 // Assign op->operand (which is a binding location) to TOS.
 // RA_TODO All these assigns could be conflated, but then will lose Dchecks.
 
@@ -255,7 +321,7 @@ case JIT_as_i_i:
 
 case JIT_subas_r_r1_r:
     DCheckSubas(TOS1, TOS, TOS2, REALSXP, REALSXP, REALSXP, op->result, p);
-    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(TOS1, r, p);
+    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(LENGTH(TOS1), r, p);
     xr = REAL(TOS1); xr[(int)r] = REAL(TOS2)[0]; // update x
     REAL(op->result)[0] = REAL(TOS2)[0];         // update return value
     drop2(p); TOS = op->result;
@@ -263,7 +329,7 @@ case JIT_subas_r_r1_r:
 
 case JIT_subas_r_r1_i:
     DCheckSubas(TOS1, TOS, TOS2, REALSXP, REALSXP, INTSXP, op->result, p);
-    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(TOS1, r, p);
+    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(LENGTH(TOS1), r, p);
     xr = REAL(TOS1); xr[(int)r] = INTEGER(TOS2)[0];
     REAL(op->result)[0] = INTEGER(TOS2)[0];
     drop2(p); TOS = op->result;
@@ -271,7 +337,7 @@ case JIT_subas_r_r1_i:
 
 case JIT_subas_r_i1_r:
     DCheckSubas(TOS1, TOS, TOS2, REALSXP, INTSXP, REALSXP, op->result, p);
-    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(TOS1, i, p);
+    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(LENGTH(TOS1), i, p);
     xr = REAL(TOS1); xr[i] = REAL(TOS2)[0];
     REAL(op->result)[0] = REAL(TOS2)[0];
     drop2(p); TOS = op->result;
@@ -279,7 +345,7 @@ case JIT_subas_r_i1_r:
 
 case JIT_subas_r_i1_i:
     DCheckSubas(TOS1, TOS, TOS2, REALSXP, INTSXP,INTSXP, op->result, p);
-    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(TOS1, i, p);
+    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(LENGTH(TOS1), i, p);
     xr = REAL(TOS1); xr[i] = INTEGER(TOS2)[0];
     REAL(op->result)[0] = INTEGER(TOS2)[0];
     drop2(p); TOS = op->result;
@@ -287,7 +353,7 @@ case JIT_subas_r_i1_i:
 
 case JIT_subas_i_r1_r:
     DCheckSubas(TOS1, TOS, TOS2, INTSXP,REALSXP, REALSXP, op->result, p);
-    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(TOS1, r, p);
+    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(LENGTH(TOS1), r, p);
     xi = INTEGER(TOS1); xi[(int)r] = REAL(TOS2)[0];
     INTEGER(op->result)[0] = (int)(REAL(TOS2)[0]);
     drop2(p); TOS = op->result;
@@ -295,7 +361,7 @@ case JIT_subas_i_r1_r:
 
 case JIT_subas_i_r1_i:
     DCheckSubas(TOS1, TOS, TOS2, INTSXP,REALSXP, INTSXP, op->result, p);
-    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(TOS1, r, p);
+    r = REAL(TOS)[0] - 1; CheckSubscriptRange_r(LENGTH(TOS1), r, p);
     xi = INTEGER(TOS1); xi[(int)r] = INTEGER(TOS2)[0];
     INTEGER(op->result)[0] = INTEGER(TOS2)[0];
     drop2(p); TOS = op->result;
@@ -303,7 +369,7 @@ case JIT_subas_i_r1_i:
 
 case JIT_subas_i_i1_r:
     DCheckSubas(TOS1, TOS, TOS2, INTSXP,INTSXP,REALSXP, op->result, p);
-    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(TOS1, i, p);
+    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(LENGTH(TOS1), i, p);
     xi = INTEGER(TOS1); xi[i] = REAL(TOS2)[0];
     INTEGER(op->result)[0] = (int)(REAL(TOS2)[0]);
     drop2(p); TOS = op->result;
@@ -311,7 +377,7 @@ case JIT_subas_i_i1_r:
 
 case JIT_subas_i_i1_i:
     DCheckSubas(TOS1, TOS, TOS2, INTSXP,INTSXP,INTSXP, op->result, p);
-    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(TOS1, i, p);
+    i = INTEGER(TOS)[0] - 1; CheckSubscriptRange_i(LENGTH(TOS1), i, p);
     xi = INTEGER(TOS1); xi[i] = INTEGER(TOS2)[0];
     INTEGER(op->result)[0] = INTEGER(TOS2)[0];
     drop2(p); TOS = op->result;
