@@ -1032,7 +1032,9 @@ static R_INLINE void assertSxpInfoZero(SEXP s) // play safe
 {
     Dassert(TYPEOF(s) == LANGSXP);
     assert(s->sxpinfo.obj == 0);
+#if 0 // RA_TODO had to remove the following for R 2.8.0 --- that worries me!
     assert(s->sxpinfo.named == 0);
+#endif
     assert(s->sxpinfo.gp == 0);
     // assert(s->sxpinfo.mark == 0);
     if (s->sxpinfo.debug)
@@ -1676,26 +1678,46 @@ void genjitLogDone(CSEXP call)
 
 // Note that LOGICAL indices are not jitted, so in evalJit JIT_subset_x
 // can create the C index by simply subtracting 1 from the R index.
-// RA_TODO This routine could be made more efficient I think.
 
 void genjitSubset(CSEXP x, CSEXP subset)    // for rhs expressions like x[i]
 {
-    static const JIT_OPCODE subsetOps[4][2] =
+    static const JIT_OPCODE subsetOps[] =
     {
-        { JIT_subset_r_r,  JIT_subset_r_r1 },
-        { JIT_subset_r_i,  JIT_subset_r_i1 },
-        { JIT_subset_i_r,  JIT_subset_i_r1 },
-        { JIT_subset_i_i,  JIT_subset_i_i1 },
+        JIT_subset_r_r1,
+        JIT_subset_r_i1,
+        JIT_subset_i_r1,
+        JIT_subset_i_i1,
     };
-    if ((TYPEOF(x) == INTSXP || TYPEOF(x) == REALSXP) &&
-            (TYPEOF(subset) == INTSXP || TYPEOF(subset) == REALSXP)) {
-        const int icol = (LENGTH(subset) == 1);
-        const int irow = 2 * (TYPEOF(x) == INTSXP) + (TYPEOF(subset) == INTSXP);
-        JIT_OPCODE opcode = subsetOps[irow][icol];
-        if (icol == 1) {  // RA_TODO for now, evalJit only supports len 1 subs
-            genjit(opcode, RNIL, NULL, NULL, 0,
-                   NILSXP, LENGTH(subset), x, RNIL, RNIL);
-        }
+    assert(TYPEOF(x) == INTSXP || TYPEOF(x) == REALSXP);
+    assert(TYPEOF(subset) == INTSXP || TYPEOF(subset) == REALSXP);
+    assert(LENGTH(subset) == 1);
+    const int icode = 2 * (TYPEOF(x) == INTSXP) + (TYPEOF(subset) == INTSXP);
+    JIT_OPCODE opcode = subsetOps[icode];
+    genjit(opcode, RNIL, NULL, NULL, 0, NILSXP, 1, x, RNIL, RNIL);
+}
+
+void genjitSubset2(CSEXP x, CSEXP subset) // for rhs expressions like x[i,j]
+{
+    static const JIT_OPCODE subsetOps[] =
+    {
+        JIT_subset2_r_r1_r1, JIT_subset2_r_r1_i1,
+        JIT_subset2_r_i1_r1, JIT_subset2_r_i1_i1,
+        JIT_subset2_i_r1_r1, JIT_subset2_i_r1_i1,
+        JIT_subset2_i_i1_r1, JIT_subset2_i_i1_i1,
+    };
+    if (TYPEOF(x) == INTSXP || TYPEOF(x) == REALSXP) {
+        assert(2 == length(getAttrib(x, R_DimSymbol)));
+	CSEXP index1 = CAR(subset);
+	CSEXP index2 = CADR(subset);
+	if ((TYPEOF(index1) == INTSXP || TYPEOF(index1) == REALSXP) &&
+		(TYPEOF(index2) == INTSXP || TYPEOF(index2) == REALSXP) &&
+		LENGTH(index2) == 1 && LENGTH(index2) == 1) {
+	    const int icode = 4 * (TYPEOF(x) == INTSXP) + 
+			      2 * (TYPEOF(index1) == INTSXP) + 
+			      (TYPEOF(index2) == INTSXP);
+	    JIT_OPCODE opcode = subsetOps[icode];
+	    genjit(opcode, RNIL, NULL, NULL, 0, NILSXP, 1, x, RNIL, RNIL);
+	}
     }
 }
 
