@@ -19,8 +19,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -52,7 +50,6 @@ void attribute_hidden nl_Rdummy(void)
 }
 #endif
 
-
 /* The 'real' main() program is in Rmain.c on Unix-alikes, and
    src/gnuwin/front-ends/graphappmain.c on Windows, unless of course
    R is embedded */
@@ -93,6 +90,7 @@ static void R_ReplFile(FILE *fp, SEXP rho, int savestack, int browselevel)
 	case PARSE_OK:
 	    R_Visible = FALSE;
 	    R_EvalDepth = 0;
+	    resetTimeLimits();
 	    count++;
 	    PROTECT(R_CurrentExpr);
 	    R_CurrentExpr = eval(R_CurrentExpr, rho);
@@ -253,6 +251,7 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
 	}
 	R_Visible = FALSE;
 	R_EvalDepth = 0;
+	resetTimeLimits();
 	PROTECT(R_CurrentExpr);
 	R_Busy(1);
         /* RA_TODO added PROTECT below, report as a bug */
@@ -355,6 +354,7 @@ int R_ReplDLLdo1(void)
 	R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 1, &status);
 	R_Visible = FALSE;
 	R_EvalDepth = 0;
+	resetTimeLimits();
 	PROTECT(R_CurrentExpr);
 	R_Busy(1);
 	R_CurrentExpr = eval(R_CurrentExpr, rho);
@@ -752,9 +752,7 @@ void setup_Rmainloop(void)
 
     InitTempDir(); /* must be before InitEd */
     InitMemory();
-#ifdef USE_CHAR_HASHING
     InitStringHash(); /* must be before InitNames */
-#endif
     InitNames();
     InitBaseEnv();
     InitGlobalEnv();
