@@ -62,11 +62,11 @@ static SEXP ExtractSubset(SEXP x, SEXP result, SEXP indx, SEXP call)
 	    ii--;
 	switch (mode) {
 	case LGLSXP:
-            if (0 <= ii && ii < nx && ii != NA_LOGICAL)
-                LOGICAL(result)[i] = LOGICAL(x)[ii];
-            else
-                LOGICAL(result)[i] = NA_INTEGER;
-            break;
+	    if (0 <= ii && ii < nx && ii != NA_LOGICAL)
+		LOGICAL(result)[i] = LOGICAL(x)[ii];
+	    else
+		LOGICAL(result)[i] = NA_INTEGER;
+	    break;
 	case INTSXP:
 	    if (0 <= ii && ii < nx && ii != NA_INTEGER)
 		INTEGER(result)[i] = INTEGER(x)[ii];
@@ -120,7 +120,7 @@ static SEXP ExtractSubset(SEXP x, SEXP result, SEXP indx, SEXP call)
 		RAW(result)[i] = (Rbyte) 0;
 	    break;
 	default:
-	    errorcall(call, R_MSG_ob_nonsub);
+	    errorcall(call, R_MSG_ob_nonsub, type2char(mode));
 	}
     }
     return result;
@@ -187,7 +187,7 @@ static SEXP VectorSubset(SEXP x, SEXP s, SEXP call)
 	nattrib = ExtractSubset(attrib, nattrib, indx, call);
 	setAttrib(result, R_SrcrefSymbol, nattrib);
 	UNPROTECT(1);
-    }    
+    }
     UNPROTECT(3);
     return result;
 }
@@ -206,6 +206,11 @@ static SEXP MatrixSubset(SEXP x, SEXP s, SEXP call, int drop)
     /* The following ensures that pointers remain protected. */
     dim = getAttrib(x, R_DimSymbol);
 
+    /* we call genjit before changing the type of s so
+       we can generate the correct opcode */
+    if (jitCompiling())
+	genjitSubset2(x, s);
+
     sr = SETCAR(s, arraySubscript(0, CAR(s), dim, getAttrib,
 				  (STRING_ELT), x));
     sc = SETCADR(s, arraySubscript(1, CADR(s), dim, getAttrib,
@@ -262,8 +267,8 @@ static SEXP MatrixSubset(SEXP x, SEXP s, SEXP call, int drop)
 		iijj = ii + jj * nr;
 		switch (TYPEOF(x)) {
 		case LGLSXP:
-                    LOGICAL(result)[ij] = LOGICAL(x)[iijj];
-                    break;
+		    LOGICAL(result)[ij] = LOGICAL(x)[iijj];
+		    break;
 		case INTSXP:
 		    INTEGER(result)[ij] = INTEGER(x)[iijj];
 		    break;
@@ -477,7 +482,7 @@ static SEXP ArraySubset(SEXP x, SEXP s, SEXP call, int drop)
 			ExtractSubset(VECTOR_ELT(dimnames, i),
 				      allocVector(STRSXP, bound[i]),
 				      CAR(r), call));
-		} else { /* 0-length dims have NULL dimnames */ 
+		} else { /* 0-length dims have NULL dimnames */
 		    SET_VECTOR_ELT(xdims, j++, R_NilValue);
 		}
 		r = CDR(r);
@@ -522,12 +527,12 @@ static SEXP ExtractArg(SEXP args, SEXP arg_sym)
 
     for (arg = prev_arg = args; arg != R_NilValue; arg = CDR(arg)) {
 	if(TAG(arg) == arg_sym) {
-            if (arg == prev_arg) /* found at head of args */
-                args = CDR(args);
-            else
-                SETCDR(prev_arg, CDR(arg));
-            found = 1;
-            break;
+	    if (arg == prev_arg) /* found at head of args */
+		args = CDR(args);
+	    else
+		SETCDR(prev_arg, CDR(arg));
+	    found = 1;
+	    break;
 	}
 	else  prev_arg = arg;
     }
@@ -597,11 +602,6 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	SEXP x = CAR(args);
 	SEXP s = CADR(args);
 	if (ATTRIB(x) == R_NilValue && ATTRIB(s) == R_NilValue) {
-            if (jitCompiling()) {
-                PROTECT(args);
-                genjitSubset(x, s);
-                UNPROTECT(1);
-            }
 	    switch (TYPEOF(x)) {
 	    case REALSXP:
 		switch (TYPEOF(s)) {
@@ -609,8 +609,14 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 		case INTSXP: i = (LENGTH(s) == 1) ? INTEGER(s)[0] : -1; break;
 		default:  i = -1;
 		}
-		if (i >= 1 && i <= LENGTH(x))
+		if (i >= 1 && i <= LENGTH(x)) {
+		    if (jitCompiling()) {
+			PROTECT(args);
+			genjitSubset(x, s);
+			UNPROTECT(1);
+		    }
 		    return ScalarReal( REAL(x)[i-1] );
+                }
 		break;
 	    case INTSXP:
 		switch (TYPEOF(s)) {
@@ -618,8 +624,14 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 		case INTSXP: i = (LENGTH(s) == 1) ? INTEGER(s)[0] : -1; break;
 		default:  i = -1;
 		}
-		if (i >= 1 && i <= LENGTH(x))
+		if (i >= 1 && i <= LENGTH(x)) {
+		    if (jitCompiling()) {
+			PROTECT(args);
+			genjitSubset(x, s);
+			UNPROTECT(1);
+		    }
 		    return ScalarInteger( INTEGER(x)[i-1] );
+                }                   
 		break;
 	    default: break;
 	    }
@@ -665,7 +677,7 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	for(px = x, i = 0 ; px != R_NilValue ; px = CDR(px))
 	    SET_VECTOR_ELT(ax, i++, CAR(px));
     }
-    else errorcall(call, R_MSG_ob_nonsub);
+    else errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
 
     /* This is the actual subsetting code. */
     /* The separation of arrays and matrices is purely an optimization. */
@@ -675,7 +687,7 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	int ndim = length(dim);
 	ans = VectorSubset(ax, (nsubs == 1 ? CAR(subs) : R_MissingArg), call);
 	/* one-dimensional arrays went through here, and they should
-	   have their dimensions dropped only if the result has 
+	   have their dimensions dropped only if the result has
 	   length one and drop == TRUE
 	*/
 	if(ndim == 1) {
@@ -691,7 +703,7 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 		if((attrib = getAttrib(x, R_DimNamesSymbol)) != R_NilValue) {
 		    /* reinstate dimnames, include names of dimnames */
 		    PROTECT(nattrib = duplicate(attrib));
-		    SET_VECTOR_ELT(nattrib, 0, 
+		    SET_VECTOR_ELT(nattrib, 0,
 				   getAttrib(ans, R_NamesSymbol));
 		    setAttrib(ans, R_DimNamesSymbol, nattrib);
 		    setAttrib(ans, R_NamesSymbol, R_NilValue);
@@ -772,9 +784,9 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
      */
     exact = ExtractExactArg(args);
     if (exact == -1)
-        pok = exact;
+	pok = exact;
     else
-        pok = !exact;
+	pok = !exact;
 
     x = CAR(args);
 
@@ -809,16 +821,16 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
       } else {
 	    SET_NAMED(ans, 2);
       }
-      
+
       UNPROTECT(1);
       if(ans == R_UnboundValue )
 	  return(R_NilValue);
       return(ans);
     }
-    
+
     /* back to the regular program */
     if (!(isVector(x) || isList(x) || isLanguage(x)))
-	errorcall(call, R_MSG_ob_nonsub);
+	errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
 
     if(nsubs == 1) { /* vector indexing */
 	SEXP thesub = CAR(subs);
@@ -967,7 +979,7 @@ SEXP attribute_hidden do_subset3(SEXP call, SEXP op, SEXP args, SEXP env)
     else if(isString(nlist) )
 	SET_STRING_ELT(input, 0, STRING_ELT(nlist, 0));
     else {
-	errorcall(call,_("invalid subscript type '%s'"), 
+	errorcall(call,_("invalid subscript type '%s'"),
 		  type2char(TYPEOF(nlist)));
     }
 
@@ -1064,9 +1076,9 @@ SEXP attribute_hidden R_subset3_dflt(SEXP x, SEXP input, SEXP call)
 	    case PARTIAL_MATCH:
 		havematch++;
 		if (havematch == 1) {
-		    /* partial matches can cause aliasing in eval.c:evalseq 
-                       This is overkill, but alternative ways to prevent 
-                       the aliasing appear to be even worse */
+		    /* partial matches can cause aliasing in eval.c:evalseq
+		       This is overkill, but alternative ways to prevent
+		       the aliasing appear to be even worse */
 		    y = VECTOR_ELT(x,i);
 		    SET_NAMED(y,2);
 		    SET_VECTOR_ELT(x,i,y);
@@ -1099,26 +1111,28 @@ SEXP attribute_hidden R_subset3_dflt(SEXP x, SEXP input, SEXP call)
 	return R_NilValue;
     }
     else if( isEnvironment(x) ){
-      	y = findVarInFrame(x, install(translateChar(input)));
-      	if( TYPEOF(y) == PROMSXP ) {
+	y = findVarInFrame(x, install(translateChar(input)));
+	if( TYPEOF(y) == PROMSXP ) {
 	    PROTECT(y);
 	    y = eval(y, R_GlobalEnv);
 	    UNPROTECT(1);
-      	}   	
-        UNPROTECT(2);      
-        if( y != R_UnboundValue ) {
-            if (NAMED(x) > NAMED(y))
-	    	SET_NAMED(y, NAMED(x));
+	}
+	UNPROTECT(2);
+	if( y != R_UnboundValue ) {
+	    if (NAMED(x) > NAMED(y))
+		SET_NAMED(y, NAMED(x));
 	    return(y);
 	}
       return R_NilValue;
     }
     else if( isVectorAtomic(x) ){
-        errorcall(call, "$ operator is invalid for atomic vectors");
+	errorcall(call, "$ operator is invalid for atomic vectors");
     }
     else if( IS_S4_OBJECT(x) ){
-        errorcall(call, "$ operator not defined for this S4 class");
-    }
+	errorcall(call, "$ operator not defined for this S4 class");
+    } 
+    else /* e.g. a function */
+	errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
     UNPROTECT(2);
     return R_NilValue;
 }
